// Fichier carre.pro

//On inclut les indices des géométries :
Include "param.geo";

/*
Création des Groupes, des entités géométriques. Ici nous n'avons besoin que du Carré intérieur, la frontière ne nous est pas utile (problème de Neumann)
*/
Group{
Carre = Region[{CarreInt}];
}

// Création de la fonction f(x,y)

Function{
// Pi est une valeur innée de GetDp
Coef = Pi;
f[] = (1+2*Coef*Coef)*Cos[Coef*X[]]*Cos[Coef*Y[]];
w[]=Cos[Coef*X[]]*Cos[Coef*Y[]];
Dw[]=Vector[-Coef*Sin[Coef*X[]]*Cos[Coef*Y[]],-Coef*Sin[Coef*Y[]]*Cos[Coef*X[]],0];
}

/*
Pour définir une fonction globalement, on note
f[] = ...
On ne met pas les arguments entre crochets mais le domaine de définition de la fonction. Par exemple nous pourrions nous restreindre au carré en notant :
f[Carre] = (1+2*Coef*Coef)*Cos[Coef*X[]]*Cos[Coef*Y[]]

Si on ne précise pas de domaine dans les crochets, la fonction est alors définie partout
*/

// Il n'y a pas de Contrainte à définir (problème de Neumann), nous passons directement à l'espace fonctionnel approchant H^1(Omega).


FunctionSpace{
  { Name Vh; Type Form0;
    BasisFunction{
      {Name wn; NameOfCoef vn; Function BF_Node;
    Support Carre; Entity NodesOf[All];}
    }
  }
}

/*
L'espace approché est appelé "Vh". Il est de type "Form0", c'est à dire scalaire.
Les fonctions de Base sont appelées "wn", et les coefficients associés "vn", c'est à dire qu'une fonction "v" de "Vh" s'écrit
v(x,y) = sum_{n} vn.wn(x,y)
Les fonctions "wn" sont des fonctions nodales P1 ("BF_Node"), de support le Carré ("Support Carre").
*/

//Importation des paramètres d'intégration et du jacobien.
Include "JacobInt.pro";

/*========================
Formulation du problème.
========================*/

Formulation{
  {Name LaplacienCarre; Type FemEquation;
// Nous appelons la formulation du problème "LaplacienCarre", qui est de type "FemEquation" (Finite Element Method Equation)
    Quantity{
      {Name u; Type Local; NameOfSpace Vh;}
      }
//On définit une quantité appelé u, qui appartient à l'espace fonctionnel Vh, défini plus haut.
    Equation{
      Galerkin{ [Dof{Grad u}, {Grad u}];
        In Carre; Jacobian JVol; Integration I1;}

      Galerkin{ [  Dof{u}, {u}];
       In Carre; Jacobian JVol; Integration I1;}
     
      Galerkin{ [-f[], {u}];
        In Carre; Jacobian JVol; Integration I1;}
    }
  }
}
/*
Entre les accolades suivant "Equation" se trouve la formulation variationnelle. Décomposons terme à terme :
_ "Galerkin" : syntaxe de GetDP, signifie mathématiquement "intégrale". (cf. en dessous).
_ Dof{u} permet de préciser que {u} est ici l'inconnue (souvent appélée "u" sur le papier).
_ Si {u} n'est pas précédé de "Dof", alors {u} fait référence à une fonction teste (souvent appelé "v" sur le papier)
(L'inconnue de les fonctions testes ont le même nom, mais nous les distinguons par le terme "Dof")
Regardons maintenant plus précisément la portion de code contenue dans les accolades suivant "Equation{ ... }" :
          Galerkin{ [Dof{Grad u}, {Grad u}];
          In Carre; Jacobian JVol; Integration Int;}
Cela signifie mathématiquement :
            Intégrale sur "Carre" de Grad(u).Grad(v)
en utilisant le Jacobien JVol et le nombre de points d'intégration défini dans "I1" (plus haut).

_ Par défaut, entre deux "Galerkin", un signe "+" est implicite
_ Par défaut aussi, la somme des intégrales "Galerkin" est égale à 0 (penser au signe "moins" pour le second membre)

Note sur le Jacobien "Jvol" : Le problème est en deux dimensions sur un domaine de deux dimensions (le carré), on utilise alors "JVol". Si l'intégrale s'effectuait sur le bord, on utiliserait "JSur".
*/

/*=======================
 Résolution du problème
=======================*/

Resolution{
  {Name LaplacienCarre;
// Nous donnons le nom LaplacienCarre à la résolution.
// (Dans GetDP, un nom est donné à toutes les entités)
    System{
      {Name Sys_Ele; NameOfFormulation LaplacienCarre;}
    }
// un système est lié à une formulation variationnelle.
// Ici nous n'en avons qu'une "LaplacienCarre"
    Operation{
      Generate[Sys_Ele]; Solve[Sys_Ele]; SaveSolution[Sys_Ele];
    }
// Les opérations que l'on effectue sont : génération du système, résolution, et sauvegarde de la solution.
  }
}


/*=============
Post Processing
=============*/

PostProcessing{
  {Name LaplaceCarre; NameOfFormulation LaplacienCarre;
    Quantity{
      {Name u; Value {Local{[{u}];In Carre;Jacobian JVol;}}}
      {Name Du; Value {Local{[{Grad u}];In Carre;Jacobian JVol;}}}
      {Name intu ; Value { Integral { [ {u}^2 ] ; In Carre ; Jacobian JVol; Integration I1 ; } } }
      {Name intDu ; Value { Integral { [ {Grad u}*{Grad u} ] ; In Carre ; Jacobian JVol; Integration I1 ; } } }
      {Name v; Value {Local{[Log10[Fabs[{u}-w[]]]];In Carre;Jacobian JVol;}}}
      {Name errl2aucarre ; Value { Integral { [ ({u}-w[])^2 ] ; In Carre ; Jacobian JVol; Integration I1 ; } } }
      {Name errh1aucarre ; Value { Integral { [ ({ Grad u}-Dw[])*({ Grad u}-Dw[])] ; In Carre ; Jacobian JVol; Integration I1 ; } } }
    }
  }
}

/*
Le Post Processing est appelé "LaplaceCarre", il appelle la formulation "LaplacienCarre", prend la solution "u" et le calcul dans le Carre par l'opération.
    Value {Local{[{u}];In Carre; Jacobian JVol;}}}
Cela signifie u est l'interpolation obtenue sur Carre, grâce à la solution {u} calculé dans la formulation LaplacienCarre.
Notons que le "u" utilisé dans PostProcessing, est le "u" qui apparaitra dans Gmsh.
Si par exemple u représente la température, nous pouvons l'appeler "Temperature" pour que GMSH affiche Temperature.
*/

/*=============
Post Operation
=============*/

PostOperation{
  {Name Map_u; NameOfPostProcessing LaplaceCarre;
    Operation{
      Print[u, OnElementsOf Carre, File "Laplacienex1.pos"];
      Print[v, OnElementsOf Carre, File "Laplacien_erreur.pos"];
      Print[ v, OnLine {{0,0,0}{1,0,0}} {100}, File "v.txt" ];

    }
  }
}

/* Ajout EC 300312 */
PostOperation{
  { Name Calcul_Normes_L2 ; NameOfPostProcessing LaplaceCarre; 
    Operation {  
      // Integral of u^2 over Carre
      Print [ intu[Carre], OnGlobal ] ;
      // Integral of Grad u * Grad u over Carre
      Print [ intDu[Carre], OnGlobal ] ;
      // Integral of error L2^2 over Carre
      Print[ errl2aucarre[Carre] , OnGlobal ] ;
      // Integral of error H1^2 over Carre
      Print[ errh1aucarre[Carre] , OnGlobal ] ;
    }
  }
}
/* Fin Ajout EC 300312 */

/*
En post Opération nous imposons juste l'affichage de "u" défini dans le Post Processing.
La post Operation est appelée "Map_u".
*/
